package edu.upvehu.gbg.docencia.perfectos;

/**
 * Realiza el cálculo de números perfectos mediante 5 aproximaciones.
 * De utilidad para ver cómo el planteamiento de algoritmos adecuados es importante,
 * y para ver las posibilidades que da Java para realizar cómputos complejos.
 * 
 * Cada ejecución se limita al cálculo de una cantidad de números "razonable" para que la ejecución se realice sin una espera excesiva 
 * (20 segs la suma de los 4 primeros métodos y 2 min el quinto en un i7_870 -8 hilos- a 2.93GHz).
 * 
 * @author german
 */
public class PruebasConNumerosPerfectos {
    private PruebasConNumerosPerfectos(){}
    
    //Topes para los distintos métodos probados.
    private static final int TOPE_CON_SUMAS=50_000; 
    private static final int MAXRO_LONG=32; //Máximo ro para la capacidad de los long
    private static final int MAXRO_BIGINTEGER=4423; //Para llegar al perfecto nº 20 con un tiempo de espera de 3 seg. en un i9-12900
    private static final int MAXRO_PARALLEL=30_000_000; //Para no parar nunca...
    //Factor de certeza para el cálculo de primalidad con BigIntegers
    private static final int CERTAINTY=8;
    //Número de hilos a usar en el pool por el método paralelo
    private static final int NUMERO_DE_HILOS=Runtime.getRuntime().availableProcessors()-2;
    
    /**
     * Realiza el cálculo de números perfectos mediante 5 aproximaciones.
     * 
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        //Para el control de tiempos de ejecución
        long elapsedTimeControl;

        // METODOS BASADOS EN LA SUMA DE LOS DIVISORES PROPIOS
        
        //Naif
        System.out.format("Probando método naif hasta %,d\n",TOPE_CON_SUMAS);
        elapsedTimeControl=System.currentTimeMillis();
        PerfectosConSumas.perfectosNaif(TOPE_CON_SUMAS);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        //Mejor (al obtener un divisor recoge tambien su co-divisor)
        System.out.format("Probando método mejor hasta %,d\n",TOPE_CON_SUMAS);
        elapsedTimeControl=System.currentTimeMillis();
        PerfectosConSumas.perfectosMejor(TOPE_CON_SUMAS);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        // METODOS BASADOS EN LOS NÚMEROS DE MERSENNE
        
        //Bueno
        System.out.format("Probando método bueno hasta la potencia %,d\n",MAXRO_LONG);
        elapsedTimeControl=System.currentTimeMillis();
        PerfectosConMersenne.perfectosBueno(MAXRO_LONG);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");
        
        //Big (usa BigInteger)
        System.out.format("Probando con uso de BigInteger hasta la potencia ro= %,d\n"
                         +"Certeza de primalidad: 1-1/2^%d = %f\n",MAXRO_BIGINTEGER,CERTAINTY,(1-1.0/(1<<CERTAINTY))); 
        elapsedTimeControl=System.currentTimeMillis();
        PerfectosConMersenne.perfectosBig(MAXRO_BIGINTEGER,CERTAINTY);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        //Threaded (usa hilos)
        System.out.format("Probando método bueno hasta la potencia ro= %,d\n"
                         +"Certeza de primalidad: 1-1/2^%d = %f\n"
                         +"Ejecutando con %d hilos\n",MAXRO_PARALLEL,CERTAINTY,(1-1.0/(1<<CERTAINTY)),NUMERO_DE_HILOS); 
        PerfectosConMersenne.perfectosBigThreaded(MAXRO_PARALLEL,CERTAINTY,NUMERO_DE_HILOS);
        System.out.println("----------------\n\n\n");

    }
    
}
